'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.replaceIndexKeys = exports.isNullUndefinedOrEmptyString = exports.reportNulls = exports.cleanNulls = exports.isBasicObject = exports.cleanFields = exports.docToModifier = undefined;

var _underscore = require('underscore');

var _underscore2 = _interopRequireDefault(_underscore);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Taken from aldeed's autoform
var docToModifier = exports.docToModifier = function docToModifier(doc, options) {
  var modifier = {};
  var mDoc;
  var flatDoc;
  var nulls;
  options = options || {};
  mDoc = new MongoObject(doc);
  flatDoc = mDoc.getFlatObject({
    keepArrays: !!options.keepArrays
  });
  nulls = reportNulls(flatDoc, !!options.keepEmptyStrings);
  nulls = cleanFields(nulls, options.fields);
  flatDoc = cleanNulls(flatDoc, false, !!options.keepEmptyStrings);
  flatDoc = cleanFields(flatDoc, options.fields);

  if (!_underscore2.default.isEmpty(flatDoc)) {
    modifier.$set = flatDoc;
  }

  if (!_underscore2.default.isEmpty(nulls)) {
    modifier.$unset = nulls;
  }

  return modifier;
};

var cleanFields = exports.cleanFields = function cleanFields(doc, fields) {
  var newDoc = {};
  _underscore2.default.each(doc, function (val, key) {
    var keys = key.split('.').reduce(function (memo, value) {
      var last = _underscore2.default.last(memo);
      var pre = last ? last + '.' : '';
      return _underscore2.default.union(memo, [pre + value]);
    }, []);
    var contains = false;
    keys.forEach(function (newKey) {
      if (_underscore2.default.contains(fields, newKey)) {
        contains = true;
      }
    });
    if (contains) {
      newDoc[key] = val;
    }
  });

  return newDoc;
};

var isBasicObject = exports.isBasicObject = function isBasicObject(obj) {
  return _underscore2.default.isObject(obj) && Object.getPrototypeOf(obj) === Object.prototype;
};

var cleanNulls = exports.cleanNulls = function cleanNulls(doc, isArray, keepEmptyStrings) {
  var newDoc = isArray ? [] : {};
  _underscore2.default.each(doc, function (val, key) {
    if (!_underscore2.default.isArray(val) && isBasicObject(val)) {
      val = cleanNulls(val, false, keepEmptyStrings); // Recurse into plain objects
      if (!_underscore2.default.isEmpty(val)) {
        newDoc[key] = val;
      }
    } else if (_underscore2.default.isArray(val)) {
      val = cleanNulls(val, true, keepEmptyStrings); // Recurse into non-typed arrays
      if (!_underscore2.default.isEmpty(val)) {
        newDoc[key] = val;
      }
    } else if (!isNullUndefinedOrEmptyString(val)) {
      newDoc[key] = val;
    } else if (keepEmptyStrings && typeof val === 'string' && val.length === 0) {
      newDoc[key] = val;
    }
  });

  return newDoc;
};

var reportNulls = exports.reportNulls = function reportNulls(flatDoc, keepEmptyStrings) {
  var nulls = {};

  // Loop through the flat doc
  _underscore2.default.each(flatDoc, function (val, key) {
    // If value is undefined, null, or an empty string, report this as null so it will be unset
    if (val === null) {
      nulls[key] = '';
    } else if (val === void 0) {
      nulls[key] = '';
    } else if (!keepEmptyStrings && typeof val === 'string' && val.length === 0) {
      nulls[key] = '';
    } else if (_underscore2.default.isArray(val) && cleanNulls(val, true, keepEmptyStrings).length === 0) {
      // If value is an array in which all the values recursively are undefined, null, or an empty string, report this as null so it will be unset
      nulls[key] = '';
    }
  });

  return nulls;
};

var isNullUndefinedOrEmptyString = exports.isNullUndefinedOrEmptyString = function isNullUndefinedOrEmptyString(val) {
  return val === void 0 || val === null || typeof val === 'string' && val.length === 0;
};

var replaceIndexKeys = exports.replaceIndexKeys = function replaceIndexKeys(key) {
  return key.replace(/\.[0-9]+\./g, '.$.');
};